#include <match_str.h>
#include <s_util.h>

proc_print_out		print_out;
proc_print_raw		print_raw;
proc_matcher_add	add_matcher;

void *string_validate(char *token, p_kv opts)
{
	p_str_priv priv;
	p_kv k;
	
	if ( !token ) return NULL;
	if ( !(priv=malloc(sizeof(str_priv_t))) ) return NULL;
	
	priv->depth=0;
	priv->offset=0;
	
	if ( !(priv->str=strdup(token)) ) {
		free(priv);
		return NULL;
	}

	for ( k=opts; k; k=(p_kv)k->l.next) {
		if ( !strcasecmp(k->key, "DEPTH") ) {
			priv->depth = atoi(k->val);
			if ( !priv->depth ) return 0;
			continue;
		}
		
		if ( !strcasecmp(k->key, "OFFSET") ) {
			priv->offset = atoi(k->val);
			if ( !priv->offset ) return 0;
			continue;
		}
		
		if ( !strcasecmp(k->key, "NOCASE") ) {
			priv->nocase = 1;
			continue;
		}
	}
	
	return priv;
}

void string_cleanup(void *p)
{
	p_str_priv priv=p;
	
	if ( !priv ) return;

	if ( priv->str ) free(priv->str);
	
	free(priv);
}

int string_match(void *priv, p_packet p, int negate)
{
	p_str_priv sp=priv;
	char *str, *end;
	int hlen=0, nlen=0, slen=0;
	int i;
	
	if ( !(str=p->layer[LAYER_APP].h.raw) ) return 0;

	hlen = ( p->caplen - ((int)str - (int)&p->data) );
	nlen = strlen(sp->str);
	
	
	/* Depth ovverides slen */
	if ( sp->depth ) {
		slen=sp->depth;
	}else{
		slen=(hlen-nlen);
		str+=sp->offset;
		slen-=sp->offset;
	}
	
	if ( slen <= 0 ) return 0;
	
	end = &p->data + p->caplen - nlen;
	
	for (i=0; i<slen; i++, str++) {
		if ( str > end ) break;
		if ( memcmp(sp->str, str, nlen) == 0 ) {
			MATCH_RET_POSITIVE;
		}
	}

	MATCH_RET_NEGATIVE;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);

	print_out=in->print_out;
	print_raw=in->print_raw;
	add_matcher=in->add_matcher;

	out->name="string matcher";
	out->desc="Matches strings in packets";
	out->ver_major=1;
	out->ver_minor=0;
	
	add_matcher("CONTENT",
		string_validate,
		string_cleanup,
		string_match);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
